extends SSTVEncoder
class_name RobotBW8

var preparedImage: Array

const BLACK = 1500.0
const WHITE = 2300.0
const COLOR_FREQ_MULT = 3.1372549

const SCAN_LINE_LENGTH = 0.056
const SYNC_PULSE_LENGTH = 0.01
const BLANKING_INTERVAL = 0.000572

const IMAGE_WIDTH = 160
const IMAGE_HEIGHT = 120

func PrepareImage(image: Image):
	image.resize(IMAGE_WIDTH, IMAGE_HEIGHT)
	image.convert(Image.FORMAT_RGB8)
	
	for h in range(image.get_height()):
		var line = PackedFloat32Array()
		for w in range(image.get_width()):
			var color = image.get_pixel(w, h)
			line.push_back(round((color.r8 + color.g8 + color.b8) / 3) * COLOR_FREQ_MULT + BLACK)
		preparedImage.push_back(line)

func EncodeSSTV(image: Image, skipHeader: bool = false):
	PrepareImage(image)
	
	if not skipHeader:
		EncodePrefix()
		EncodeHeader([false, false, false, false, false, true, false])
	
	for scanLine in range(0, IMAGE_HEIGHT):
		outputFloats.append_array(GenerateTone(SYNC_PULSE_FREQ, SYNC_PULSE_LENGTH))
		outputFloats.append_array(GenerateTone(BLANKING_PULSE_FREQ, BLANKING_INTERVAL))
		outputFloats.append_array(GenerateToneFromCurve(preparedImage[scanLine], SCAN_LINE_LENGTH))
		outputFloats.append_array(GenerateTone(BLANKING_PULSE_FREQ, BLANKING_INTERVAL))
		
	return outputFloats
